@@ -58,6 +58,7 @@ from chex._src.asserts import assert_trees_all_equal
 from chex._src.asserts import assert_trees_all_equal_comparator
 from chex._src.asserts import assert_trees_all_equal_dtypes
 from chex._src.asserts import assert_trees_all_equal_shapes
+from chex._src.asserts import assert_trees_all_equal_shapes_and_dtypes
 from chex._src.asserts import assert_trees_all_equal_structs
 from chex._src.asserts import assert_type
 from chex._src.asserts import clear_trace_counter
@@ -66,6 +67,7 @@ from chex._src.asserts import enable_asserts
 from chex._src.asserts import if_args_not_none
 from chex._src.asserts_chexify import block_until_chexify_assertions_complete
 from chex._src.asserts_chexify import chexify
+from chex._src.asserts_chexify import ChexifyChecks
 from chex._src.asserts_chexify import with_jittable_assertions
 from chex._src.dataclass import dataclass
 from chex._src.dataclass import mappable_dataclass
@@ -96,7 +98,7 @@ from chex._src.variants import TestCase
 from chex._src.variants import variants
 
 
-__version__ = "0.1.5"
+__version__ = "0.1.6"
 
 __all__ = (
     "all_variants",
@@ -107,6 +109,7 @@ __all__ = (
     "ArrayNumpy",
     "ArraySharded",
     "ArrayTree",
+    "ChexifyChecks",
     "assert_axis_dimension",
     "assert_axis_dimension_comparator",
     "assert_axis_dimension_gt",
@@ -151,6 +154,7 @@ __all__ = (
     "assert_trees_all_equal_comparator",
     "assert_trees_all_equal_dtypes",
     "assert_trees_all_equal_shapes",
+    "assert_trees_all_equal_shapes_and_dtypes",
     "assert_trees_all_equal_structs",
     "assert_type",
     "block_until_chexify_assertions_complete",
