@@ -143,7 +143,7 @@ def assert_max_traces(fn: Optional[Union[Callable[..., Any], int]] = None,
 
     nonlocal fn_hash
     _ai.TRACE_COUNTER[fn_hash] += int(has_tracers_in_args)
-    if _ai.TRACE_COUNTER[fn_hash] > n:
+    if not _ai.DISABLE_ASSERTIONS and _ai.TRACE_COUNTER[fn_hash] > n:
       raise AssertionError(
           f"{_ai.ERR_PREFIX}Function '{fn.__name__}' is traced > {n} times!\n"
           "It often happens when a jitted function is defined inside another "
@@ -541,9 +541,10 @@ def assert_shape(
       match; if ``expected_shapes`` has wrong type; if shape of ``input`` does
       not match ``expected_shapes``.
   """
-  if isinstance(expected_shapes, (np.ndarray, Array)):
-    raise AssertionError("Error in shape compatibility check: "
-                         "expected shapes should be a list or tuple of ints.")
+  if not isinstance(expected_shapes, (list, tuple)):
+    raise AssertionError(
+        "Error in shape compatibility check: expected shapes should be a list "
+        f"or tuple of ints, got {expected_shapes}.")
 
   # Ensure inputs and expected shapes are sequences.
   if not isinstance(inputs, collections.abc.Sequence):
@@ -645,10 +646,16 @@ def assert_rank(
     ValueError: If ``expected_ranks`` is not an integer and not a sequence of
      integets.
   """
-  if isinstance(expected_ranks, (np.ndarray, Array)):
+  if not isinstance(expected_ranks, (collections.abc.Collection, int)):
     raise ValueError(
-        f"Error in rank compatibility check: expected ranks should "
-        f"be a collection of integers but was an array: {expected_ranks}.")
+        f"Error in rank compatibility check: expected ranks should be a single "
+        f"integer or a collection of integers, got {expected_ranks}.")
+
+  if isinstance(expected_ranks, np.ndarray):  # ndarray is abc.Collection
+    raise ValueError(
+        f"Error in rank compatibility check: expected ranks should be a single "
+        f"integer or a collection of integers, but was an array: "
+        f"{expected_ranks}.")
 
   # Ensure inputs and expected ranks are sequences.
   if not isinstance(inputs, collections.abc.Sequence):
@@ -926,7 +933,8 @@ def assert_tree_no_nones(tree: ArrayTree) -> None:
       nonlocal errors
       errors.append(f"`None` detected at '{_ai.format_tree_path(path)}'.")
 
-  dm_tree.map_structure_with_path(_assert_fn, tree)
+  for path, leaf in dm_tree.flatten_with_path(tree):
+    _assert_fn(path, leaf)
   if errors:
     raise AssertionError("\n".join(errors))
 
@@ -956,11 +964,25 @@ def assert_tree_has_only_ndarrays(tree: ArrayTree,
         errors.append((f"Tree leaf '{_ai.format_tree_path(path)}' is not an "
                        f"ndarray (type={type(leaf)})."))
 
-  dm_tree.map_structure_with_path(_assert_fn, tree)
+  for path, leaf in dm_tree.flatten_with_path(tree):
+    _assert_fn(path, leaf)
   if errors:
     raise AssertionError("\n".join(errors))
 
 
+# Only look the sharding attribute after jax version >= 0.3.22 i.e. remove this
+# function and use `isinstance(x.sharding, jax.sharding.PmapSharding)` after
+# jax version >= 0.3.22.
+# This is for backwards compatibility.
+def _check_sharding(x):
+  if jax.config.jax_array and isinstance(x, jax.Array):
+    if isinstance(x.sharding, jax.sharding.PmapSharding):
+      return True
+    else:
+      return len(x.sharding.device_set) > 1
+  return isinstance(x, jax.pxla.ShardedDeviceArray)
+
+
 @_static_assertion
 def assert_tree_is_on_host(tree: ArrayTree,
                            *,
@@ -987,8 +1009,7 @@ def assert_tree_is_on_host(tree: ArrayTree,
       if not isinstance(leaf, np.ndarray):
         nonlocal errors
 
-        if (isinstance(leaf, jax.xla.DeviceArray) and
-            not isinstance(leaf, jax.pxla.ShardedDeviceArray)):
+        if isinstance(leaf, jax.Array) and not _check_sharding(leaf):
           if allow_cpu_device:
             if leaf.device().platform != "cpu":
               errors.append((f"Tree leaf '{_ai.format_tree_path(path)}' resides"
@@ -1000,7 +1021,8 @@ def assert_tree_is_on_host(tree: ArrayTree,
           errors.append((f"Tree leaf '{_ai.format_tree_path(path)}' has "
                          f"unexpected type: {type(leaf)}."))
 
-  dm_tree.map_structure_with_path(_assert_fn, tree)
+  for path, leaf in dm_tree.flatten_with_path(tree):
+    _assert_fn(path, leaf)
   if errors:
     raise AssertionError("\n".join(errors))
 
@@ -1043,8 +1065,8 @@ def assert_tree_is_on_device(tree: ArrayTree,
       nonlocal errors
 
       # Check that the leaf is a DeviceArray.
-      if isinstance(leaf, jax.xla.DeviceArray):
-        if isinstance(leaf, jax.pxla.ShardedDeviceArray):
+      if isinstance(leaf, jax.Array):
+        if _check_sharding(leaf):
           errors.append((f"Tree leaf '{_ai.format_tree_path(path)}' is a "
                          f"ShardedDeviceArray which are disallowed. "
                          f" (type={type(leaf)})."))
@@ -1064,7 +1086,8 @@ def assert_tree_is_on_device(tree: ArrayTree,
         errors.append((f"Tree leaf '{_ai.format_tree_path(path)}' has "
                        f"unexpected type: {type(leaf)}."))
 
-  dm_tree.map_structure_with_path(_assert_fn, tree)
+  for path, leaf in dm_tree.flatten_with_path(tree):
+    _assert_fn(path, leaf)
   if errors:
     raise AssertionError("\n".join(errors))
 
@@ -1096,16 +1119,17 @@ def assert_tree_is_sharded(tree: ArrayTree,
       nonlocal errors
 
       # Check that the leaf is a ShardedArray.
-      if not isinstance(leaf, jax.pxla.ShardedDeviceArray):
-        errors.append((f"Tree leaf '{_ai.format_tree_path(path)}' is not a "
-                       f"ShardedDeviceArray (type={type(leaf)})."))
-      else:
+      if isinstance(leaf, jax.Array) and _check_sharding(leaf):
         shards = tuple(buf.device() for buf in leaf.device_buffers)
         if shards != devices:
           errors.append((f"Tree leaf '{_ai.format_tree_path(path)}' is sharded "
                          f"across {shards} devices, expected {devices}."))
+      else:
+        errors.append((f"Tree leaf '{_ai.format_tree_path(path)}' is not a "
+                       f"ShardedDeviceArray (type={type(leaf)})."))
 
-  dm_tree.map_structure_with_path(_assert_fn, tree)
+  for path, leaf in dm_tree.flatten_with_path(tree):
+    _assert_fn(path, leaf)
   if errors:
     raise AssertionError("\n".join(errors))
 
@@ -1134,7 +1158,7 @@ def assert_tree_shape_prefix(tree: ArrayTree,
 
   errors = []
 
-  def _assert_prefix_fn(path, leaf):
+  def _assert_fn(path, leaf):
     if leaf is None:
       return
 
@@ -1153,7 +1177,8 @@ def assert_tree_shape_prefix(tree: ArrayTree,
           (f"Tree leaf '{_ai.format_tree_path(path)}' has a shape prefix "
            f"different from expected: {suffix} != {shape_prefix}."))
 
-  dm_tree.map_structure_with_path(_assert_prefix_fn, tree)
+  for path, leaf in dm_tree.flatten_with_path(tree):
+    _assert_fn(path, leaf)
   if errors:
     raise AssertionError("\n".join(errors))
 
@@ -1181,7 +1206,7 @@ def assert_tree_shape_suffix(tree: ArrayTree,
 
   errors = []
 
-  def _assert_suffix_fn(path, leaf):
+  def _assert_fn(path, leaf):
     if leaf is None:
       return
 
@@ -1200,13 +1225,14 @@ def assert_tree_shape_suffix(tree: ArrayTree,
           (f"Tree leaf '{_ai.format_tree_path(path)}' has a shape suffix "
            f"different from expected: {suffix} != {shape_suffix}."))
 
-  dm_tree.map_structure_with_path(_assert_suffix_fn, tree)
+  for path, leaf in dm_tree.flatten_with_path(tree):
+    _assert_fn(path, leaf)
   if errors:
     raise AssertionError("\n".join(errors))
 
 
 @_static_assertion
-def assert_trees_all_equal_structs(*trees: Sequence[ArrayTree]) -> None:
+def assert_trees_all_equal_structs(*trees: ArrayTree) -> None:
   """Checks that trees have the same structure.
 
   Note that `None` is treated as a PyTree node.
@@ -1243,7 +1269,7 @@ assert_tree_all_equal_structs = _ai.deprecation_wrapper(
 @_static_assertion
 def assert_trees_all_equal_comparator(equality_comparator: _ai.TLeavesEqCmpFn,
                                       error_msg_fn: _ai.TLeavesEqCmpErrorFn,
-                                      *trees: Sequence[ArrayTree],
+                                      *trees: ArrayTree,
                                       ignore_nones: bool = False) -> None:
   """Checks that all trees are equal as per the custom comparator for leaves.
 
@@ -1288,9 +1314,14 @@ def assert_trees_all_equal_comparator(equality_comparator: _ai.TLeavesEqCmpFn,
 
     return equality_comparator(leaf_1, leaf_2)
 
-  cmp = functools.partial(_ai.assert_leaves_all_eq_comparator,
-                          wrapped_equality_comparator, tree_error_msg_fn)
-  dm_tree.map_structure_with_path(cmp, *trees)
+  cmp_fn = functools.partial(_ai.assert_leaves_all_eq_comparator,
+                             wrapped_equality_comparator, tree_error_msg_fn)
+
+  # Trees are guaranteed to have the same structure.
+  paths = [path for path, _ in dm_tree.flatten_with_path(trees[0])]
+  trees_leaves = [dm_tree.flatten(tree) for tree in trees]
+  for leaf_i, path in enumerate(paths):
+    cmp_fn(path, *[leaves[leaf_i] for leaves in trees_leaves])
 
 
 assert_tree_all_equal_comparator = _ai.deprecation_wrapper(
@@ -1300,7 +1331,7 @@ assert_tree_all_equal_comparator = _ai.deprecation_wrapper(
 
 
 @_static_assertion
-def assert_trees_all_equal_dtypes(*trees: Sequence[ArrayTree],
+def assert_trees_all_equal_dtypes(*trees: ArrayTree,
                                   ignore_nones: bool = False) -> None:
   """Checks that trees' leaves have the same dtype.
 
@@ -1330,7 +1361,7 @@ def assert_trees_all_equal_dtypes(*trees: Sequence[ArrayTree],
 
 
 @_static_assertion
-def assert_trees_all_equal_shapes(*trees: Sequence[ArrayTree],
+def assert_trees_all_equal_shapes(*trees: ArrayTree,
                                   ignore_nones: bool = False) -> None:
   """Checks that trees have the same structure and leaves' shapes.
 
@@ -1353,6 +1384,25 @@ assert_tree_all_equal_shapes = _ai.deprecation_wrapper(
     old_name="assert_tree_all_equal_shapes",
     new_name="assert_trees_all_equal_shapes")
 
+
+@_static_assertion
+def assert_trees_all_equal_shapes_and_dtypes(
+    *trees: ArrayTree, ignore_nones: bool = False) -> None:
+  """Checks that trees' leaves have the same shape and dtype.
+
+  Note that `None` is treated as a PyTree nodes.
+
+  Args:
+    *trees: A sequence of (at least 2) trees to check.
+    ignore_nones: Whether to ignore `None` in the trees.
+
+  Raises:
+    AssertionError: If leaves' shapes or dtypes for any two trees differ.
+  """
+  assert_trees_all_equal_shapes(*trees, ignore_nones=ignore_nones)
+  assert_trees_all_equal_dtypes(*trees, ignore_nones=ignore_nones)
+
+
 ############# Value assertions. #############
 
 
@@ -1381,12 +1431,14 @@ def _assert_tree_all_finite_jittable(tree_like: ArrayTree) -> Array:
       ]))
 
 
-assert_tree_all_finite = _value_assertion(_assert_tree_all_finite_static,
-                                          _assert_tree_all_finite_jittable)
+assert_tree_all_finite = _value_assertion(
+    assert_fn=_assert_tree_all_finite_static,
+    jittable_assert_fn=_assert_tree_all_finite_jittable,
+    name="assert_tree_all_finite")
 
 
 @_static_assertion
-def _assert_trees_all_equal_static(*trees: Sequence[ArrayTree],
+def _assert_trees_all_equal_static(*trees: ArrayTree,
                                    ignore_nones: bool = False) -> None:
   """Checks that all trees have leaves with *exactly* equal values.
 
@@ -1428,7 +1480,7 @@ def _assert_trees_all_equal_static(*trees: Sequence[ArrayTree],
       cmp_fn, err_msg_fn, *trees, ignore_nones=ignore_nones)
 
 
-def _assert_trees_all_equal_jittable(*trees: Sequence[ArrayTree],
+def _assert_trees_all_equal_jittable(*trees: ArrayTree,
                                      ignore_nones: bool = False) -> Array:
   """A jittable version of `_assert_trees_all_equal_static`."""
   if not ignore_nones:
@@ -1438,11 +1490,13 @@ def _assert_trees_all_equal_jittable(*trees: Sequence[ArrayTree],
   return _ai.assert_trees_all_eq_comparator_jittable(cmp_fn, *trees)
 
 
-assert_trees_all_equal = _value_assertion(_assert_trees_all_equal_static,
-                                          _assert_trees_all_equal_jittable)
+assert_trees_all_equal = _value_assertion(
+    assert_fn=_assert_trees_all_equal_static,
+    jittable_assert_fn=_assert_trees_all_equal_jittable,
+    name="assert_trees_all_equal")
 
 
-def _assert_trees_all_close_static(*trees: Sequence[ArrayTree],
+def _assert_trees_all_close_static(*trees: ArrayTree,
                                    rtol: float = 1e-06,
                                    atol: float = .0,
                                    ignore_nones: bool = False) -> None:
@@ -1491,7 +1545,7 @@ def _assert_trees_all_close_static(*trees: Sequence[ArrayTree],
       cmp_fn, err_msg_fn, *trees, ignore_nones=ignore_nones)
 
 
-def _assert_trees_all_close_jittable(*trees: Sequence[ArrayTree],
+def _assert_trees_all_close_jittable(*trees: ArrayTree,
                                      rtol: float = 1e-06,
                                      atol: float = .0,
                                      ignore_nones: bool = False) -> Array:
@@ -1503,8 +1557,10 @@ def _assert_trees_all_close_jittable(*trees: Sequence[ArrayTree],
   return _ai.assert_trees_all_eq_comparator_jittable(cmp_fn, *trees)
 
 
-assert_trees_all_close = _value_assertion(_assert_trees_all_close_static,
-                                          _assert_trees_all_close_jittable)
+assert_trees_all_close = _value_assertion(
+    assert_fn=_assert_trees_all_close_static,
+    jittable_assert_fn=_assert_trees_all_close_jittable,
+    name="assert_trees_all_close")
 
 assert_tree_all_close = _ai.deprecation_wrapper(
     assert_trees_all_close,
