@@ -16,10 +16,13 @@
 
 import functools
 import re
+import sys
+import threading
 import time
 from typing import Any, Optional, Sequence, Type
 
 from absl.testing import absltest
+from absl.testing import parameterized
 from chex._src import asserts
 from chex._src import asserts_chexify
 from chex._src import asserts_internal as _ai
@@ -49,21 +52,25 @@ def _assert_noop(*args,
 # Define a simple Chex assertion for testing purposes.
 def _assert_tree_positive(tree):
   # Use jnp instead of np for testing purposes.
-  if not all((x > 0).all() for x in jax.tree_leaves(tree)):
+  if not all((x > 0).all() for x in jax.tree_util.tree_leaves(tree)):
     raise AssertionError('Tree contains non-positive elems!')
 
 
 def _jittable_assert_tree_positive(tree):
   # Jittable version of `_assert_tree_positive`.
-  return jnp.all(jnp.array([(x > 0).all() for x in jax.tree_leaves(tree)]))
+  return jnp.all(
+      jnp.array([(x > 0).all() for x in jax.tree_util.tree_leaves(tree)]))
 
 
 chex_static_assert_positive = _ai.chex_assertion(
-    assert_fn=_assert_tree_positive, jittable_assert_fn=None)
+    assert_fn=_assert_tree_positive,
+    jittable_assert_fn=None,
+    name='assert_tree_positive_test')
 
 chex_value_assert_positive = _ai.chex_assertion(
     assert_fn=_assert_tree_positive,
-    jittable_assert_fn=_jittable_assert_tree_positive)
+    jittable_assert_fn=_jittable_assert_tree_positive,
+    name='assert_tree_positive_test')
 
 
 class AssertsChexifyTest(variants.TestCase):
@@ -83,7 +90,7 @@ class AssertsChexifyTest(variants.TestCase):
     # Define a simple function that uses the assertion.
     def _sum_fn(tree):
       jax.tree_map(lambda x: chex_assert_shape(x, shape), tree)
-      return sum(x.sum() for x in jax.tree_leaves(tree))
+      return sum(x.sum() for x in jax.tree_util.tree_leaves(tree))
 
     chexified_sum_fn = chexify_sync(self.variant(_sum_fn))
     # Passes in all contexts.
@@ -176,8 +183,26 @@ class AssertsChexifyTest(variants.TestCase):
     invalid_x = -jnp.ones(3)
     chexify_async(_pos_sum)(invalid_x)  # async error
 
-    with self.assertRaisesRegex(AssertionError, 'err_label'):
-      asserts_chexify.atexit._run_exitfuncs()
+    raised_exception = None
+
+    def unraisablehook(unraisable):
+      nonlocal raised_exception
+      raised_exception = unraisable.exc_value
+
+    if sys.version_info >= (3, 10):
+      # In Python 3.10+, _run_exitfuncs logs the exception using unraisablehook
+      # without raising the last exception.
+      old_unraisablehook = sys.unraisablehook
+      sys.unraisablehook = unraisablehook
+      try:
+        asserts_chexify.atexit._run_exitfuncs()
+      finally:
+        sys.unraisablehook = old_unraisablehook
+      self.assertIsInstance(raised_exception, AssertionError)
+      self.assertIn('err_label', str(raised_exception))
+    else:
+      with self.assertRaisesRegex(AssertionError, 'err_label'):
+        asserts_chexify.atexit._run_exitfuncs()
 
   def test_docstring_example(self):
 
@@ -190,11 +215,56 @@ class AssertsChexifyTest(variants.TestCase):
     logp1_abs_safe(jnp.ones(2))  # OK
     asserts_chexify.block_until_chexify_assertions_complete()
 
-    err_regex = re.escape(_ai.get_chexify_err_message())
+    err_regex = re.escape(_ai.get_chexify_err_message('assert_tree_all_finite'))
     with self.assertRaisesRegex(AssertionError, f'{err_regex}.*chexify_test'):
       logp1_abs_safe(jnp.array([jnp.nan, 3]))  # FAILS
       logp1_abs_safe.wait_checks()
 
+  def test_checkify_errors(self):
+    @jax.jit
+    def take_by_index_and_div(x, i, y):
+      return x[i] / y
+
+    # Checks only Div-by-0.
+    take_by_index_and_div_safe_div = asserts_chexify.chexify(
+        take_by_index_and_div,
+        async_check=False,
+        errors=asserts_chexify.ChexifyChecks.div,
+    )
+    take_by_index_and_div_safe_div(jnp.ones(2), 1, 2)  # OK
+    take_by_index_and_div_safe_div(jnp.ones(2), 10, 2)  # OOB undetected
+
+    with self.assertRaisesRegex(
+        asserts_chexify.checkify.JaxRuntimeError, 'division by zero'
+    ):
+      take_by_index_and_div_safe_div(jnp.ones(2), 1, 0)  # Div-by-0
+
+    # Checks both OOB and Div-by-0.
+    take_by_index_and_div_safe = asserts_chexify.chexify(
+        take_by_index_and_div,
+        async_check=False,
+        errors=(
+            asserts_chexify.ChexifyChecks.index
+            | asserts_chexify.ChexifyChecks.div
+        ),
+    )
+    take_by_index_and_div_safe(jnp.ones(2), 1, 2)  # OK
+
+    with self.assertRaisesRegex(
+        asserts_chexify.checkify.JaxRuntimeError, 'out-of-bounds'
+    ):
+      take_by_index_and_div_safe(jnp.ones(2), 10, 2)  # OOB
+
+    with self.assertRaisesRegex(
+        asserts_chexify.checkify.JaxRuntimeError, 'division by zero'
+    ):
+      take_by_index_and_div_safe(jnp.ones(2), 1, 0)  # Div-by-0
+
+    with self.assertRaisesRegex(
+        asserts_chexify.checkify.JaxRuntimeError, 'out-of-bounds'
+    ):
+      take_by_index_and_div_safe(jnp.ones(2), 10, 0)  # OOB (first) & Div-by-0
+
 
 class AssertsChexifyTestSuite(variants.TestCase):
   """Test suite for chexify assertions."""
@@ -246,7 +316,9 @@ class AssertsChexifyTestSuite(variants.TestCase):
       # Reports incorrect usage without `chexify()`.
       with self.assertRaisesRegex(
           RuntimeError, 'can only be called from functions wrapped .*chexify'):
-        jax_transform(fn_value_assert)(*valid_args)
+        # Create a local object to avoid reusing jax internal cache.
+        local_fn_value_assert = make_test_fn(chex_value_assert_positive)
+        jax_transform(local_fn_value_assert)(*valid_args)
 
     # Run tests with invalid arguments.
     for invalid_args, label in zip(all_invalid_args, failure_labels):
@@ -256,17 +328,20 @@ class AssertsChexifyTestSuite(variants.TestCase):
           fn_static_assert(*invalid_args)
 
       # Value assertion fails on incorrect inputs (with transformations).
-      err_regex = re.escape(_ai.get_chexify_err_message(label))
+      err_regex = re.escape(
+          _ai.get_chexify_err_message('assert_tree_positive_test', label))
       with self.assertRaisesRegex(AssertionError, err_regex):
         chexified_fn_with_value_asserts(*invalid_args)
 
       # Reports incorrect usage without `chexify()`.
       with self.assertRaisesRegex(
           RuntimeError, 'can only be called from functions wrapped .*chexify'):
-        jax_transform(fn_value_assert)(*invalid_args)
+        # Create a local object to avoid reusing jax internal cache.
+        local_fn_value_assert = make_test_fn(chex_value_assert_positive)
+        jax_transform(local_fn_value_assert)(*invalid_args)
 
   def run_test_suite_with_log_abs_fn(self, make_log_fn, jax_transform, devices,
-                                     run_pure):
+                                     run_pure, run_in_thread):
     """Generates valid and invalid inputs for log_abs_fn and runs the tests."""
     x_pos = {
         'a': np.ones((10, 2)),
@@ -295,10 +370,33 @@ class AssertsChexifyTestSuite(variants.TestCase):
         'label_1',
     )
 
-    self.run_test_suite(make_log_fn, all_valid_args, all_invalid_args,
-                        failure_labels, jax_transform, run_pure)
+    run_tests_callback = functools.partial(self.run_test_suite, make_log_fn,
+                                           all_valid_args, all_invalid_args,
+                                           failure_labels, jax_transform,
+                                           run_pure)
+    if run_in_thread:
+      failure = AssertionError('not executed')
 
-  def test_log_abs_fn_jitted(self):
+      def _run_tests_in_thread():
+        nonlocal failure
+        failure = None
+        try:
+          run_tests_callback()
+        except Exception as e:  # pylint:disable=broad-except
+          failure = e
+
+      thr = threading.Thread(target=_run_tests_in_thread, daemon=False)
+      thr.start()
+      thr.join(timeout=30)
+      if thr.is_alive():
+        raise TimeoutError('Thread is alive after 30 seconds.')
+      if failure is not None:
+        raise AssertionError(f'Thread failed with: {failure}.')
+    else:
+      run_tests_callback()
+
+  @parameterized.named_parameters(('main', False), ('thread', True))
+  def test_log_abs_fn_jitted(self, run_in_thread):
     """Tests simple jit transformation."""
 
     def _make_log_fn(assert_input_fn: _ai.TChexAssertion):
@@ -318,9 +416,11 @@ class AssertsChexifyTestSuite(variants.TestCase):
           make_log_fn=_make_log_fn,
           jax_transform=jax.jit,
           devices=jax.local_devices()[:1],
-          run_pure=True)
+          run_pure=True,
+          run_in_thread=run_in_thread)
 
-  def test_log_abs_fn_jitted_nested_wrap(self):
+  @parameterized.named_parameters(('main', False), ('thread', True))
+  def test_log_abs_fn_jitted_nested_wrap(self, run_in_thread):
     """Tests nested jit transforms (wrapping)."""
 
     def _make_log_fn(assert_input_fn: _ai.TChexAssertion):
@@ -345,9 +445,11 @@ class AssertsChexifyTestSuite(variants.TestCase):
           make_log_fn=_make_log_fn,
           jax_transform=jax.jit,
           devices=jax.local_devices()[:1],
-          run_pure=False)  # do not run because internal jit is not checkified
+          run_pure=False,  # do not run because internal jit is not checkified
+          run_in_thread=run_in_thread)
 
-  def test_log_abs_fn_jitted_nested_call(self):
+  @parameterized.named_parameters(('main', False), ('thread', True))
+  def test_log_abs_fn_jitted_nested_call(self, run_in_thread):
     """Tests nested jit transforms (calling)."""
 
     def _make_log_fn(assert_input_fn: _ai.TChexAssertion):
@@ -371,9 +473,11 @@ class AssertsChexifyTestSuite(variants.TestCase):
           make_log_fn=_make_log_fn,
           jax_transform=jax.jit,
           devices=jax.local_devices()[:1],
-          run_pure=False)  # do not run because internal jit is not checkified
+          run_pure=False,  # do not run because internal jit is not checkified
+          run_in_thread=run_in_thread)
 
-  def test_log_abs_fn_pmapped(self):
+  @parameterized.named_parameters(('main', False), ('thread', True))
+  def test_log_abs_fn_pmapped(self, run_in_thread):
     """Tests pmap transform."""
 
     def _make_log_fn(assert_input_fn: _ai.TChexAssertion):
@@ -394,9 +498,11 @@ class AssertsChexifyTestSuite(variants.TestCase):
           make_log_fn=_make_log_fn,
           jax_transform=lambda fn: jax.pmap(fn, axis_name='i'),
           devices=jax.local_devices(),
-          run_pure=False)  # do not run because the f-n contains collective ops
+          run_pure=False,  # do not run because the f-n contains collective ops
+          run_in_thread=run_in_thread)
 
-  def test_log_abs_fn_jitted_vmapped(self):
+  @parameterized.named_parameters(('main', False), ('thread', True))
+  def test_log_abs_fn_jitted_vmapped(self, run_in_thread):
     """Tests vmap transform."""
 
     def _make_log_fn(assert_input_fn: _ai.TChexAssertion):
@@ -416,7 +522,8 @@ class AssertsChexifyTestSuite(variants.TestCase):
           make_log_fn=_make_log_fn,
           jax_transform=lambda fn: jax.jit(jax.vmap(fn)),  # jax + vmap
           devices=jax.local_devices()[:1],
-          run_pure=True)
+          run_pure=True,
+          run_in_thread=run_in_thread)
 
 
 if __name__ == '__main__':
