@@ -52,21 +52,26 @@ TLeavesEqCmpErrorFn = Callable[[TLeaf, TLeaf], str]
 #  **kwargs)
 TChexAssertion = Callable[..., None]
 TAssertFn = Callable[..., None]
-TJittableAssertFn = Callable[..., bool]  # a predicate function
+TJittableAssertFn = Callable[..., pytypes.Array]  # a predicate function
 
 # Matchers.
 TDimMatcher = Optional[Union[int, Set[int], type(Ellipsis)]]
 TShapeMatcher = Sequence[TDimMatcher]
 
+
+class _ChexifyStorage(threading.local):
+  """Thread-safe storage for internal variables used in @chexify."""
+  wait_fns = []
+  level = 0
+
+
 # Chex namespace variables.
 ERR_PREFIX = "[Chex] "
 TRACE_COUNTER = collections.Counter()
 DISABLE_ASSERTIONS = False
 
 # This variable is used for _chexify_ transformations, see `asserts_chexify.py`.
-CHEXIFY_STORAGE = threading.local()
-CHEXIFY_STORAGE.wait_fns = []
-CHEXIFY_STORAGE.level = 0
+CHEXIFY_STORAGE = _ChexifyStorage()
 
 
 def assert_collection_of_arrays(inputs: Sequence[pytypes.Array]):
@@ -75,16 +80,12 @@ def assert_collection_of_arrays(inputs: Sequence[pytypes.Array]):
     raise ValueError(f"`inputs` is not a collection of arrays: {inputs}.")
 
 
-def jnp_to_np_array(arr: pytypes.Array) -> pytypes.Array:
+def jnp_to_np_array(arr: pytypes.Array) -> np.ndarray:
   """Converts `jnp.ndarray` to `np.ndarray`."""
-  if isinstance(arr, jnp.ndarray):
-    if arr.dtype == jnp.bfloat16:
-      # Numpy does not support `bfloat16`.
-      return np.asarray(arr, np.float32)
-    else:
-      return np.asarray(arr, arr.dtype)
-  else:
-    return arr
+  if getattr(arr, "dtype", None) == jnp.bfloat16:
+    # Numpy does not support `bfloat16`.
+    arr = arr.astype(jnp.float32)
+  return jax.device_get(arr)
 
 
 def deprecation_wrapper(new_fn, old_name, new_name):
@@ -128,13 +129,14 @@ def get_err_regex(message: str) -> str:
   return f"{re.escape(ERR_PREFIX)}[\\s\\S]*{message}"
 
 
-def get_chexify_err_message(custom_msg: Optional[str] = None) -> str:
+def get_chexify_err_message(name: str, custom_msg: Optional[str] = None) -> str:
   """Constructs an error message for the chexify exception."""
   custom_msg = f" [{custom_msg}]" if custom_msg else ""
-  return f"{ERR_PREFIX}chexify assertion failed{custom_msg}"
+  return f"{ERR_PREFIX}chexify assertion '{name}' failed{custom_msg}"
 
 
-def _make_host_assertion(assert_fn: TAssertFn) -> TChexAssertion:
+def _make_host_assertion(assert_fn: TAssertFn,
+                         name: Optional[str] = None) -> TChexAssertion:
   """Constructs a host assertion given `assert_fn`.
 
   This wrapper should only be applied to the assertions that are either
@@ -145,10 +147,13 @@ def _make_host_assertion(assert_fn: TAssertFn) -> TChexAssertion:
 
   Args:
     assert_fn: A function implementing the check.
+    name: A name for assertion.
 
   Returns:
     A chex assertion.
   """
+  if name is None:
+    name = assert_fn.__name__
 
   def _assert_on_host(*args,
                       custom_message: Optional[str] = None,
@@ -178,7 +183,7 @@ def _make_host_assertion(assert_fn: TAssertFn) -> TChexAssertion:
           error_msg = error_msg[error_msg.find("failed:") + len("failed:"):]
 
         # Whether to include the default error message.
-        default_msg = (f"Assertion {assert_fn.__name__} failed: "
+        default_msg = (f"Assertion {name} failed: "
                        if include_default_message else "")
         error_msg = f"{ERR_PREFIX}{default_msg}{error_msg}"
 
@@ -195,7 +200,8 @@ def _make_host_assertion(assert_fn: TAssertFn) -> TChexAssertion:
 
 def chex_assertion(
     assert_fn: TAssertFn,
-    jittable_assert_fn: Optional[TJittableAssertFn]) -> TChexAssertion:
+    jittable_assert_fn: Optional[TJittableAssertFn],
+    name: Optional[str] = None) -> TChexAssertion:
   """Wraps Chex assert functions to control their common behaviour.
 
   Extends the assertion to support the following optional auxiliary kwargs:
@@ -211,28 +217,43 @@ def chex_assertion(
     jittable_assert_fn: An optional jittable version of `assert_fn` implementing
       a predicate (returning `True` only if assertion passes).
       Required for value assertions.
+    name: A name for assertion. If not provided, use `assert_fn.__name__`.
 
   Returns:
     A Chex assertion (with auxiliary kwargs).
   """
-  host_assertion_fn = _make_host_assertion(assert_fn)
+  if name is None:
+    name = assert_fn.__name__
+
+  host_assertion_fn = _make_host_assertion(assert_fn, name)
 
   @functools.wraps(assert_fn)
-  def _chex_assert_fn(*args, **kwargs) -> None:
+  def _chex_assert_fn(*args,
+                      custom_message: Optional[str] = None,
+                      custom_message_format_vars: Sequence[Any] = (),
+                      include_default_message: bool = True,
+                      exception_type: Type[Exception] = AssertionError,
+                      **kwargs) -> None:
     if DISABLE_ASSERTIONS:
       return
-    if jittable_assert_fn is not None and has_tracers((args, kwargs)):
+    if (jittable_assert_fn is not None and has_tracers((args, kwargs))):
       if not CHEXIFY_STORAGE.level:
         raise RuntimeError(
             "Value assertions can only be called from functions wrapped "
             "with `@chex.chexify`. See the docs.")
-      msg = get_chexify_err_message(kwargs.pop("custom_message", None))
+      msg = get_chexify_err_message(name, custom_message)
       callsite_frame = get_last_non_chex_frame()
       msg += f" [failed at {callsite_frame.filename}:{callsite_frame.lineno}]"
       checkify.check(pred=jittable_assert_fn(*args, **kwargs), msg=msg)
     else:
       try:
-        host_assertion_fn(*args, **kwargs)
+        host_assertion_fn(
+            *args,
+            custom_message=custom_message,
+            custom_message_format_vars=custom_message_format_vars,
+            include_default_message=include_default_message,
+            exception_type=exception_type,
+            **kwargs)
       except jax.errors.ConcretizationTypeError as exc:
         msg = ("Chex assertion detected `ConcretizationTypeError`: it is very "
                "likely that it tried to access tensors' values during tracing. "
@@ -240,6 +261,8 @@ def chex_assertion(
                "assertion; if that does not help, please file a bug.")
         raise exc from RuntimeError(msg)
 
+  # Override name.
+  setattr(_chex_assert_fn, "__name__", name)
   return _chex_assert_fn
 
 
@@ -297,11 +320,14 @@ def is_traceable(fn) -> bool:
       "pmap.",  # Python pmap
       "PmapFunction",  # C++ pmap in jaxlib 0.1.72 or newer.
       "vmap.",  # vmap
+      "_python_pjit",
+      "_cpp_pjit",
   )
 
   fn_type_tokens = (
       "CompiledFunction",
       "PmapFunction",
+      "PjitFunction",
   )
 
   # Un-wrap `fn` and check if any internal fn is jitted by pattern matching.
@@ -326,7 +352,8 @@ def is_traceable(fn) -> bool:
         return True
 
       try:
-        if isinstance(fn_, jax.lib.xla_extension.jax_jit.CompiledFunction):
+        if isinstance(fn_, (jax.lib.xla_extension.jax_jit.CompiledFunction,
+                            jax.lib.xla_extension.PjitFunction)):
           return True
       except AttributeError:
         pass
