@@ -465,7 +465,7 @@ class RankAssertTest(parameterized.TestCase):
   def test_rank_should_fail_array_expectations(self):
     with self.assertRaisesRegex(  # pylint: disable=g-error-prone-assert-raises
         ValueError,
-        'expected ranks should be a collection of integers but was an array'):
+        'expected ranks should be .* but was an array'):
       asserts.assert_rank(rank_array(2), np.array([2]))
 
   def test_rank_should_fail_wrong_expectation_structure(self):
@@ -1032,7 +1032,7 @@ class TreeAssertionsTest(parameterized.TestCase):
       with self.assertRaisesRegex(AssertionError,
                                   _get_err_regex('\'1/a\' resides on \'cpu\'')):
         asserts.assert_tree_is_on_device((tpu_1_2_tree, cpu_tree),
-                                         platform=('tpu'))
+                                         platform='tpu')
       with self.assertRaisesRegex(AssertionError,
                                   _get_err_regex('\'0/a\' resides on \'tpu\'')):
         asserts.assert_tree_is_on_device((tpu_1_2_tree, cpu_tree),
@@ -1191,8 +1191,8 @@ class TreeAssertionsTest(parameterized.TestCase):
     self._assert_tree_structs_validation(asserts.assert_trees_all_close)
 
   def test_assert_trees_all_close_fails_values_differ(self):
-    tree1 = (jnp.array([0.0, 2.0]))
-    tree2 = (jnp.array([0.0, 2.1]))
+    tree1 = jnp.array([0.0, 2.0])
+    tree2 = jnp.array([0.0, 2.1])
     asserts.assert_trees_all_close(tree1, tree2, atol=0.1)
     with self.assertRaisesRegex(
         AssertionError, _get_err_regex('Values not approximately equal')):
@@ -1325,6 +1325,29 @@ class TreeAssertionsTest(parameterized.TestCase):
                                 _get_err_regex('Trees 0 and 1 differ')):
       asserts.assert_trees_all_equal_dtypes(t_5, t_6)
 
+  def test_assert_trees_all_equal_shapes_and_dtypes(self):
+    # Test dtypes
+    t_0 = {'x': np.zeros(3, dtype=np.int16), 'y': np.ones(2, dtype=np.float32)}
+    t_1 = {'x': np.zeros(3, dtype=np.uint16), 'y': np.ones(2, dtype=np.float32)}
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('Trees 0 and 1 differ')):
+      asserts.assert_trees_all_equal_shapes_and_dtypes(t_0, t_1)
+
+    t_2 = {'x': np.zeros(3, dtype=np.int16), 'y': np.ones(2, dtype=np.float32)}
+    asserts.assert_trees_all_equal_shapes_and_dtypes(t_0, t_2, t_0)
+
+    # Test shapes
+    t_0 = {'x': np.zeros(3, dtype=np.int16), 'y': np.ones(2, dtype=np.float32)}
+    t_1 = {'x': np.zeros(4, dtype=np.int16), 'y': np.ones(2, dtype=np.float32)}
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('Trees 0 and 1 differ')):
+      asserts.assert_trees_all_equal_shapes_and_dtypes(t_0, t_1)
+
+    t_2 = {'x': np.zeros(3, dtype=np.int16), 'y': np.ones(2, dtype=np.float32)}
+    asserts.assert_trees_all_equal_shapes_and_dtypes(t_0, t_2, t_0)
+
   def test_assert_trees_all_equal_wrong_usage(self):
     # not an array
     with self.assertRaisesRegex(AssertionError,
