@@ -20,6 +20,7 @@ import functools
 
 from absl import logging
 import jax
+from typing_extensions import dataclass_transform  # pytype: disable=not-supported-yet
 
 
 FrozenInstanceError = dataclasses.FrozenInstanceError
@@ -82,6 +83,7 @@ def mappable_dataclass(cls):
   return cls
 
 
+@dataclass_transform()
 def dataclass(
     cls=None,
     *,
@@ -226,6 +228,18 @@ class _Dataclass():
     return dcls
 
 
+def _dataclass_unflatten(dcls, keys, values):
+  dcls_object = dcls.__new__(dcls)
+  attribute_dict = dict(zip(keys, values))
+  # Looping over fields instead of keys & values preserves the field order.
+  # Using dataclasses.fields fails because dataclass uids change after
+  # serialisation (eg, with cloudpickle).
+  for field in dcls.__dataclass_fields__.values():
+    if field.name in attribute_dict:  # Filter pseudo-fields.
+      object.__setattr__(dcls_object, field.name, attribute_dict[field.name])
+  return dcls_object
+
+
 def register_dataclass_type_with_jax_tree_util(data_class):
   """Register an existing dataclass so JAX knows how to handle it.
 
@@ -240,7 +254,7 @@ def register_dataclass_type_with_jax_tree_util(data_class):
       in instance.__dict__.
   """
   flatten = lambda d: jax.util.unzip2(sorted(d.__dict__.items()))[::-1]
-  unflatten = lambda keys, values: data_class(**dict(zip(keys, values)))
+  unflatten = functools.partial(_dataclass_unflatten, data_class)
   try:
     jax.tree_util.register_pytree_node(
         nodetype=data_class, flatten_func=flatten, unflatten_func=unflatten)
