@@ -72,7 +72,10 @@ def _assert_pmapped(fn, fn_input, is_pmapped, should_jit=False):
   # the function output, if it is a sharded array type then the function has
   # been pmapped
   if is_pmapped:
-    expected_type = ArraySharded
+    if jax.config.jax_array:
+      expected_type = jax.Array
+    else:
+      expected_type = ArraySharded
     assert_message = f'Output is type {type(output)}, expected {expected_type}'
     assert isinstance(output, expected_type), assert_message
   else:
@@ -80,8 +83,12 @@ def _assert_pmapped(fn, fn_input, is_pmapped, should_jit=False):
     assert_message = f'Output is type {type(output)}, expected {expected_type}'
     # ShardedDeviceArray is a subclass of DeviceArray. So, to enforce we have
     # a DeviceArray, we also check it's not a sharded one.
-    assert (isinstance(output, jnp.DeviceArray) and
-            not isinstance(output, ArraySharded)), assert_message
+    if jax.config.jax_array:
+      assert (isinstance(output, jax.Array) and
+              len(output.sharding.device_set) == 1), assert_message
+    else:
+      assert (isinstance(output, jax.Array) and
+              not isinstance(output, ArraySharded)), assert_message
 
 
 class PmapFakeTest(parameterized.TestCase):
@@ -92,8 +99,13 @@ class PmapFakeTest(parameterized.TestCase):
     fn_input = jnp.ones((4,))
 
     _assert_pmapped(foo, fn_input, True)
-    with self.assertRaises(AssertionError):
-      _assert_pmapped(foo, fn_input, False)
+    # Since this test runs only on 1 device, having a test to check if the
+    # output is sharded or not is not correct. With jax.Array, you can check
+    # the `len(output.sharding.device_set)` to see if its sharded or not, but
+    # here because of a single device it fails.
+    if not jax.config.jax_array:
+      with self.assertRaises(AssertionError):
+        _assert_pmapped(foo, fn_input, False)
 
   def test_assert_jitted(self):
     fn_input = jnp.ones((4,))
