@@ -282,7 +282,7 @@ class OneFailedVariantTest(variants.TestCase):
 
       @self.variant
       def fails_for_without_device_variant(x):
-        self.assertIsInstance(x, jnp.DeviceArray)
+        self.assertIsInstance(x, jax.Array)
 
       fails_for_without_device_variant(42)
 
@@ -653,8 +653,8 @@ class WithJitTest(parameterized.TestCase):
 
 def _test_fn_without_device(arg_0, arg_1):
   tc = unittest.TestCase()
-  tc.assertNotIsInstance(arg_0, jnp.DeviceArray)
-  tc.assertNotIsInstance(arg_1, jnp.DeviceArray)
+  tc.assertNotIsInstance(arg_0, jax.Array)
+  tc.assertNotIsInstance(arg_1, jax.Array)
   return DEFAULT_FN(arg_0, arg_1)
 
 
@@ -675,8 +675,8 @@ class WithoutDeviceTest(parameterized.TestCase):
 
 def _test_fn_with_device(arg_0, arg_1):
   tc = unittest.TestCase()
-  tc.assertIsInstance(arg_0, jnp.DeviceArray)
-  tc.assertIsInstance(arg_1, jnp.DeviceArray)
+  tc.assertIsInstance(arg_0, jax.Array)
+  tc.assertIsInstance(arg_1, jax.Array)
   return DEFAULT_FN(arg_0, arg_1)
 
 
@@ -702,7 +702,7 @@ class WithDeviceTest(parameterized.TestCase):
     @self.variant(ignore_argnums=(0, 2))
     def fn(arg_0, arg_1, float_arg):
       self.assertIsInstance(arg_0, static_type)
-      self.assertIsInstance(arg_1, jnp.DeviceArray)
+      self.assertIsInstance(arg_1, jax.Array)
       self.assertIsInstance(float_arg, float)
       return DEFAULT_FN(arg_0, arg_1)
 
